SystemOrganization addCategory: #'Zippers-Support'!
SystemOrganization addCategory: #Zippers!
SystemOrganization addCategory: #'Zippers-Tests'!

Object subclass: #ListZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ListZipper commentStamp: 'fbs 12/28/2010 16:27' prior: 0!
A zipper is a structure that allows one, in a purely functional manner, to navigate a structure. It works by remembering the steps taken to focus on any particular part of a structure.

A ListZipper is a zipper for any sequenceable collection. It has two instance variables, namely

focus - the (sub)list on which we are currently focussed.
trail - the steps taken to focus on the focus instance variable.!

!ListZipper class methodsFor: 'instance creation' stamp: 'fbs 12/28/2010 16:48'!
on: aSequenceableCollection
	^ self new focus: aSequenceableCollection asOrderedCollection trail: OrderedCollection new.! !

!ListZipper methodsFor: 'mutating' stamp: 'fbs 12/30/2010 20:50'!
change: aBlock
	"Given an arity 1 block, replace the current value in the list with that of evaluating aBlock on that value."
	
	^ ListZipper new
		focus: (aBlock value: focus first) asOrderedCollection, focus allButFirst
		trail: trail.! !

!ListZipper methodsFor: 'accessing' stamp: 'fbs 12/28/2010 16:27'!
focus
	^ focus.! !

!ListZipper methodsFor: 'private' stamp: 'fbs 12/28/2010 16:36'!
focus: aSeqColl trail: anotherSeqColl
	focus := aSeqColl.
	trail := anotherSeqColl.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
left
	trail isEmpty ifTrue: [^ ZipperNavigation new signal: 'Left at the first element'].
	
	^ ListZipper new
		focus: (focus copy addFirst: trail first; yourself)
		trail: trail allButFirst.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
right
	focus isEmpty ifTrue: [^ ZipperNavigation new signal: 'Right at the last element'].
	
	^ ListZipper new
		focus: focus allButFirst
		trail: (trail copy addFirst: focus first; yourself)! !

!ListZipper methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:24'!
trail
	^ trail.! !

Object subclass: #TreeZipper
	instanceVariableNames: 'focus trail leftNodes rightNodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipper commentStamp: 'fbs 12/30/2010 22:41' prior: 0!
I am a zipper for navigating over a variable-arity tree, that is a tree that may have any number of children per node.

In particular, Smalltalk has a built-in tree structure, that of dependents: any Object may have any number of dependents, who may each in turn have their own dependents. Really, this allows for a directed graph rather than a tree. Further, we treat objects with no dependents as leaf nodes. Every node is its own value.

That is, I allow one to navigate arbitrarily over a tree structure, and change parts of it, where "change" means "return a structure that shares part of the original structure, but otherwise leaves the host structure unchanged".!

!TreeZipper class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 22:04'!
on: aTree
	^ self new focus: aTree trail: OrderedCollection new.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:49'!
down
	"Return a Zipper focusing on the first child element of this node."
	(self focus children isEmpty)
		ifTrue: [ ^ ZipperNavigation signal: 'Down at a leaf node' ].
	^ TreeZipper new
		focus: self focus children first
		trail: (trail
			addFirst: (TreeZipperTrail
				value: focus value
				zipper: (ListZipper on: focus children));
			yourself).! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 12/30/2010 22:06'!
focus
	^ focus.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 12/30/2010 22:04'!
focus: aTree trail: anOrderedCollection
	focus := aTree.
	trail := anOrderedCollection.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 21:40'!
left! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 21:41'!
right! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:58'!
up
	| parent |
	trail isEmpty ifTrue: [^ ZipperNavigation signal: 'Up at the root node'].
	
	"Parent is the last TreeZipperTrail we used - a zipper on our father and uncles."
	parent := trail first.
	^ TreeZipper new
		focus: (ZTree
			value: parent value
			children: parent zipper focus, parent zipper trail reverse)
		trail: trail allButFirst.! !

Object subclass: #TreeZipperTrail
	instanceVariableNames: 'value zipper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipperTrail class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:46'!
value: anObject zipper: aListZipper
	^ self new value: anObject zipper: aListZipper.! !

!TreeZipperTrail methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:45'!
value
	^ value.! !

!TreeZipperTrail methodsFor: 'private' stamp: 'fbs 12/30/2010 23:47'!
value: anObject zipper: aListZipper
	value := anObject.
	zipper := aListZipper.! !

!TreeZipperTrail methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:48'!
zipper
	^ zipper.! !

Object subclass: #ZTree
	instanceVariableNames: 'children value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZTree commentStamp: 'fbs 12/30/2010 23:32' prior: 0!
I'm a basic implementation of a tree, only present so that my package doesn't need to rely on a separate tree package.!

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:40'!
value: anObject
	^ self new value: anObject children: OrderedCollection new.! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:41'!
value: anObject children: aCollection
	^ self new value: anObject children: aCollection.! !

!ZTree methodsFor: 'as yet unclassified' stamp: 'fbs 12/31/2010 00:08'!
= anObject
	^ anObject class == self class and:
		[self value = anObject value and:
			[self children = anObject children]].! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:33'!
children
	^ children! !

!ZTree methodsFor: 'initialize-release' stamp: 'fbs 12/30/2010 23:34'!
initialize
	super initialize.
	children := OrderedCollection new.! !

!ZTree methodsFor: 'printing' stamp: 'fbs 12/30/2010 23:54'!
printString
	^ '(', self value printString,
		(children
			ifEmpty: ['']
			ifNotEmpty: [' ', (self children reduce: [:a :b | a printString, ', ', b printString])]),
		')'.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:40'!
value
	^ value.! !

!ZTree methodsFor: 'private' stamp: 'fbs 12/30/2010 23:41'!
value: anObject children: aCollection
	value := anObject.
	children := aCollection asOrderedCollection.! !

TestCase subclass: #ListZipperTest
	instanceVariableNames: 'prefix suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangeChangesValue
	| list zipper |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangePreservesOriginal
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right.
	zipper := zipper change: [:x | x size].
	zipper := zipper left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.
	self assert: list = original description: 'Zipper not functional'.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:15'!
testChangeSharesStructure
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 (2 3) 5) asOrderedCollection.
	self assert: zipper focus second == original second description: 'Zipper not sharing original structure'.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testLeft
	| left list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right right.
	
	left := right left.
	self assert: left focus = list allButFirst.
	
	left := left left.
	self assert: left focus = list.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testLeftPastBeginningOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right left left ]
		raise: ZipperNavigation.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:49'!
testOn
	| list zipper |
	list := #(1 2 3 4 5).
	zipper := ListZipper on: list.
	self assert: zipper focus = list asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testRight
	| list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right.
	self assert: right focus = list allButFirst.
	
	right := right right.
	self assert: right focus = list allButFirst allButFirst.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testRightPastEndOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right right right ]
		raise: ZipperNavigation.! !

TestCase subclass: #TreeZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/30/2010 23:39'!
emptyTree
	^ ZTree value: 0.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:18'!
testDownOnEmptyTreeSignalsException
	| zipper |
	zipper := TreeZipper on: self emptyTree.
	
	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:59'!
testDownOnLeafNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 23:44'!
testDownOnNonEmptyTreeFocusesOnFirstSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down.
	
	self assert: zipper focus = tree children first.! !

!TreeZipperTest methodsFor: 'testing - instance creation' stamp: 'fbs 12/30/2010 22:19'!
testOn
	| tree zipper |
	tree := self emptyTree.
	zipper := TreeZipper on: tree.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:21'!
testUpAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper up]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 23:08'!
testUpReturnsToParentNode
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down.
	zipper := zipper up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/30/2010 23:39'!
triangleTree
	" Best viewed in a monospaced font:
	   A
	  / \
	 B   C "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

Error subclass: #ZipperNavigation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperNavigation commentStamp: 'fbs 12/28/2010 17:36' prior: 0!
Raise me whenever a Zipper tries to navigate to somewhere it cannot - past the end of a list, to a missing subtree, etc.!
